@@ -8,7 +8,6 @@
 
 
 import os
-import re
 import sys
 import imp
 import time
@@ -76,8 +75,8 @@ def freeze(dist):
     #  For win32 we include a special chainloader that can suck the selected
     #  version into the running process rather than spawn a new proc.
     code_source = ["__name__ = '__main__'"]
-    esky_name = re.escape(dist.distribution.get_name())
-    code_source.append("__esky_name__ = '%s'" % (esky_name,))
+    esky_name = dist.distribution.get_name()
+    code_source.append("__esky_name__ = %r" % (esky_name,))
     code_source.append(inspect.getsource(esky.bootstrap))
     if dist.compile_bootstrap_exes:
         if sys.platform == "win32":
