@@ -10,7 +10,6 @@ from __future__ import with_statement
 
 
 import os
-import re
 import sys
 import imp
 import time
@@ -19,6 +18,8 @@ import zipfile
 import shutil
 import tempfile
 import inspect
+import struct
+import marshal
 from StringIO import StringIO
 
 
@@ -43,23 +44,14 @@ def freeze(dist):
         excludes.append("pypy")
     options["includes"] = includes
     options["excludes"] = excludes
-    # py2app can't simultaneously freeze multiple scripts.
-    # We do a separate freeze of each then merge them together.
     # The control info (name, icon, etc) for the app will be taken from
-    # the first script in the list.
+    # the first script in the list.  Subsequent scripts will be passed
+    # as the extra_scripts argument.
     exes = list(dist.get_executables())
     if not exes:
         raise RuntimeError("no scripts specified")
-    cmd = _make_py2app_cmd(dist.freeze_dir,dist.distribution,options,exes[0])
+    cmd = _make_py2app_cmd(dist.freeze_dir,dist.distribution,options,exes)
     cmd.run()
-    for exe in exes[1:]:
-        tempdir = tempfile.mkdtemp()
-        try:
-            cmd = _make_py2app_cmd(tempdir,dist.distribution,options,exe)
-            cmd.run()
-            _merge_dir(tempdir,dist.freeze_dir)
-        finally:
-            shutil.rmtree(tempdir)
     #  Remove any .pyc files with a corresponding .py file.
     #  This helps avoid timestamp changes that might interfere with
     #  the generation of useful patches between versions.
@@ -90,8 +82,8 @@ def freeze(dist):
         lib.write(src,arcnm)
     lib.close()
     #  Create the bootstraping code, using custom code if specified.
-    esky_name = re.escape(dist.distribution.get_name())
-    code_source = ["__esky_name__ = '%s'" % (esky_name,)]
+    esky_name = dist.distribution.get_name()
+    code_source = ["__esky_name__ = %r" % (esky_name,)]
     code_source.append(inspect.getsource(esky.bootstrap))
     if not dist.compile_bootstrap_exes:
         code_source.append(_FAKE_ESKY_BOOTSTRAP_MODULE)
@@ -130,15 +122,25 @@ def freeze(dist):
                 if nm.startswith("fcntl"):
                     copy_to_bootstrap_env(os.path.join(dynload,nm))
         copy_to_bootstrap_env("Contents/Resources/__error__.sh")
-        copy_to_bootstrap_env("Contents/Resources/__boot__.py")
-        copy_to_bootstrap_env("Contents/Resources/site.py")
-        #  Copy the bootstrapping code into the __boot__.py file.
+        # Copy site.py/site.pyc into the boostrap env, then zero them out.
         bsdir = dist.bootstrap_dir
+        if os.path.exists(os.path.join(app_dir, "Contents/Resources/site.py")):
+            copy_to_bootstrap_env("Contents/Resources/site.py")
+            with open(bsdir + "/Contents/Resources/site.py", "wt") as f:
+                pass
+        if os.path.exists(os.path.join(app_dir, "Contents/Resources/site.pyc")):
+            copy_to_bootstrap_env("Contents/Resources/site.pyc")
+            with open(bsdir + "/Contents/Resources/site.pyc", "wb") as f:
+                f.write(imp.get_magic() + struct.pack("<i", 0))
+                f.write(marshal.dumps(compile("", "site.py", "exec")))
+        if os.path.exists(os.path.join(app_dir, "Contents/Resources/site.pyo")):
+            copy_to_bootstrap_env("Contents/Resources/site.pyo")
+            with open(bsdir + "/Contents/Resources/site.pyo", "wb") as f:
+                f.write(imp.get_magic() + struct.pack("<i", 0))
+        #  Copy the bootstrapping code into the __boot__.py file.
+        copy_to_bootstrap_env("Contents/Resources/__boot__.py")
         with open(bsdir+"/Contents/Resources/__boot__.py","wt") as f:
             f.write(code_source)
-        #  Clear site.py in the bootstrap dir, it doesn't do anything useful.
-        with open(bsdir+"/Contents/Resources/site.py","wt") as f:
-            f.write("")
         #  Copy the loader program for each script into the bootstrap env.
         copy_to_bootstrap_env("Contents/MacOS/python")
         for exe in dist.get_executables(normalise=False):
@@ -147,6 +149,9 @@ def freeze(dist):
             exepath = copy_to_bootstrap_env("Contents/MacOS/"+exe.name)
     #  Copy non-python resources (e.g. icons etc) into the bootstrap dir
     copy_to_bootstrap_env("Contents/Info.plist")
+    # Include Icon
+    if exe.icon is not None:
+        copy_to_bootstrap_env("Contents/Resources/"+exe.icon)
     copy_to_bootstrap_env("Contents/PkgInfo")
     with open(os.path.join(app_dir,"Contents","Info.plist"),"rt") as f:
         infotxt = f.read()
@@ -168,13 +173,15 @@ def zipit(dist,bsdir,zfname):
     return create_zipfile(bsdir,zfname,get_arcname,compress=True)
 
 
-def _make_py2app_cmd(dist_dir,distribution,options,exe):
+def _make_py2app_cmd(dist_dir,distribution,options,exes):
+    exe = exes[0]
+    extra_exes = exes[1:]
     cmd = py2app(distribution)
     for (nm,val) in options.iteritems():
         setattr(cmd,nm,val)
     cmd.dist_dir = dist_dir
-    cmd.app = [Target(script=exe.script,dest_base=exe.name,
-                      prescripts=[StringIO(_EXE_PRESCRIPT_CODE)])]
+    cmd.app = [Target(script=exe.script,dest_base=exe.name)]
+    cmd.extra_scripts = [e.script for e in extra_exes]
     cmd.finalize_options()
     cmd.plist["CFBundleExecutable"] = exe.name
     old_run = cmd.run
@@ -198,19 +205,6 @@ def _make_py2app_cmd(dist_dir,distribution,options,exe):
     return cmd
 
 
-def _merge_dir(src,dst):
-    if not os.path.isdir(dst):
-        os.makedirs(dst)
-    for nm in os.listdir(src):
-        srcnm = os.path.join(src,nm)
-        dstnm = os.path.join(dst,nm)
-        if os.path.isdir(srcnm):
-            _merge_dir(srcnm,dstnm)
-        else:
-            if not os.path.exists(dstnm):
-               shutil.copy2(srcnm,dstnm)
-        
-
 #  Code to fake out any bootstrappers that try to import from esky.
 _FAKE_ESKY_BOOTSTRAP_MODULE = """
 class __fake:
@@ -227,17 +221,3 @@ from posix import environ
 sys.executable = environ["EXECUTABLEPATH"]
 sys.argv[0] = environ["ARGVZERO"]
 """
-
-
-#  py2app isn't designed for freezing multiple exes, so its standard
-#  bootstrap code runs a fixed script.  This code gets inserted into the
-#  bootstrap code to inspect the environment and find the actual script
-#  to be run.
-_EXE_PRESCRIPT_CODE = """
-import os
-import sys
-scriptnm = os.path.basename(os.environ["EXECUTABLEPATH"])
-_run(scriptnm + ".py")
-sys.exit(0)
-"""
-
