@@ -222,6 +222,17 @@ def appdir_from_executable(exepath):
     return appdir
 
 
+def appexe_from_executable(exepath):
+    """Find the top-level application executable, given sys.executable."""
+    appdir = appdir_from_executable(exepath)
+    exename = os.path.basename(exepath)
+    #  On OSX we might be in a bundle, run from Contents/MacOS/<exename>
+    if sys.platform == "darwin":
+        if os.path.isdir(os.path.join(appdir,"Contents","MacOS")):
+            return os.path.join(appdir,"Contents","MacOS",exename)
+    return os.path.join(appdir,exename)
+
+
 def extract_zipfile(source,target,name_filter=None):
     """Extract the contents of a zipfile into a target directory.
 
@@ -450,7 +461,7 @@ def really_rename(source,target):
     if sys.platform != "win32":
         os.rename(source,target)
     else:
-        for _ in xrange(3):
+        for _ in xrange(10):
             try:
                 os.rename(source,target)
             except WindowsError, e:
@@ -478,7 +489,7 @@ def really_rmtree(path):
         if not os.path.exists(path):
             shutil.rmtree(path)
         #  This is a little retry loop that catches troublesome errors.
-        for _ in xrange(3):
+        for _ in xrange(10):
             try:
                 shutil.rmtree(path)
             except WindowsError, e:
